
/**
 * @author original c++ code: Ng Weng Leong, Java port: Ben Coleman
 *
 */

import java.awt.Color;

public class Atom
{
	
	private final Color[] COLOR_LIST = {Color.cyan, Color.green, Color.lightGray,
										Color.magenta, Color.orange, Color.pink, Color.red, Color.white, Color.yellow};
	private final double EPSILON = 1e-24;
	private final double MIN_SPEED = 1e-100;
	private final int COLOR_MODIFICATION_OPTIONS = 2;
	
	private Vec myVelocity, myPosition, oldPosition;
	private double myMass, myRadius;
	private double pathLength;
	private int numHits;
	private int id;
	private boolean isSelected;
	private Color selectedColor;
	
	public Atom(Vec velocity, Vec position, double mass, double radius, int id)
	{
		oldPosition = position;
		myVelocity  = velocity;
		myPosition = position;
		myMass = mass;
		myRadius = radius;
		pathLength = 0;
		numHits = 0;
		this.id = id;
		isSelected = false;
		selectedColor = COLOR_LIST[(int)(Math.random() * COLOR_LIST.length)];
		int lightenOrDarken = (int)(Math.random() * COLOR_MODIFICATION_OPTIONS);
		
		if(lightenOrDarken == 0)
		{
			selectedColor = selectedColor.brighter();
		}
	}
	
	public double timeToHitContainer(double radius)
	{
		double vp = myVelocity.dotProduct(myPosition), vv = myVelocity.dotProduct(myVelocity), 
			   pp = myPosition.dotProduct(myPosition);
		double squareDistanceToAtomEdge = StrictMath.pow(radius - myRadius, 2) - pp;		
		
		double timeToHit = (-vp + StrictMath.sqrt(StrictMath.pow(vp, 2) + vv * squareDistanceToAtomEdge)) / vv;
		
		if(squareDistanceToAtomEdge < -EPSILON) //guards against the possibility of an atom overshooting the edge
		{										//if this is true, timeToHit is NaN and this causes all other atoms
			timeToHit = Double.MAX_VALUE;		//to fly off
		}
		
		return timeToHit;
	}
	
	public double timeToHitAtom(Atom atm)
	{
		Vec relativeVelocity = myVelocity.subtract(atm.myVelocity), 
			relativePosition = myPosition.subtract(atm.myPosition);
		
		double vp = relativeVelocity.dotProduct(relativePosition), 
			   squareLinearRelativeVelocity = relativeVelocity.dotProduct(relativeVelocity),
			   squareDistance = relativePosition.dotProduct(relativePosition);
		
		double minTime = -vp / squareLinearRelativeVelocity;
		
		if(minTime < -EPSILON)
		{
			return Double.MAX_VALUE;
		}
		
		if(minTime < 0)
		{
			minTime = 0;
		}
		
		double df = 4 * Math.pow((myRadius + atm.myRadius) / 2, 2) - (squareDistance + vp * minTime);
		
		if(df < -EPSILON)
		{
			return Double.MAX_VALUE;
		}
		
		if(df < 0)
		{
			df = 0;
		}
		
		double timeToHit = minTime - Math.sqrt(df / StrictMath.abs(squareLinearRelativeVelocity));
		
		if(timeToHit < EPSILON)	//prevents atoms from becoming stuck together
		{						//occasionally two atoms get stuck at a distance that causes timeToHit to drop to extremely low
			timeToHit = 0;		//levels.  This prevents that from happening, but could possibly lead to valid collisions
		}						//being tossed because they just happened to be slightly less than EPSILON
								//this has not been observed yet, however
		return timeToHit;
	}
	
	public Vec getPosition()
	{
		return myPosition;
	}
	
	public Vec getOldPosition()
	{
		return oldPosition;
	}
	
	public void setPosition(Vec position)
	{
		Vec distance = position.subtract(myPosition);
		pathLength += StrictMath.sqrt(distance.dotProduct(distance));
		oldPosition = myPosition;
		myPosition = position;
	}
	
	public Vec getVelocity()
	{
		return myVelocity;
	}
	
	public void setVelocity(Vec velocity)
	{
		Vec temp = myVelocity;
		myVelocity = velocity;
		
		if(getLinearSpeed() == 0) 																							//If the linear speed is 0, then NaN is returned by timeToHitAtom
		{																													//if the atom it is going to collide with also has 0 speed
			myVelocity = temp;																								//This sets the velocity to 1e-100, a value that is low enough
			myVelocity = new Vec(StrictMath.sin(getDirection()) * MIN_SPEED, StrictMath.cos(getDirection()) * MIN_SPEED);	//to appear to be 0
		}
		
	}
	
	public double getRadius()
	{
		return myRadius;
	}
	
	public double getPathLength()
	{
		return pathLength;
	}
	
	public int getNumHits()
	{
		return numHits;
	}
	
	public void iterateNumHits()
	{
		numHits ++;
	}
	
	public int getId()
	{
		return id;
	}
	
	public double distanceTo(double x, double y)
	{
		return StrictMath.sqrt(StrictMath.pow(myPosition.getX() - x, 2) + 
				   			   StrictMath.pow(myPosition.getY() - y, 2));
	}
	
	public double getMass()
	{
		return myMass;
	}
	
	public double getLinearSpeed()
	{
		return StrictMath.sqrt(myVelocity.dotProduct(myVelocity));
	}
	
	public String toString()
	{
		return "Atom " + id + " position: " + getPosition() + " velocity: " + getVelocity() + "(m/s) mass: " + getMass() + "(kg)";
	}
	
	public void setMass(double newMass)
	{
		myMass = newMass;
	}
	
	public void setRadius(double newRadius)
	{
		myRadius = newRadius;
	}
	
	public double getDirection()
	{
		double angle = StrictMath.atan(myVelocity.getY()/myVelocity.getX());
		
		if(myVelocity.getX() <= 0)
		{
			angle += Math.PI;
		}
		
		else if(myVelocity.getY() <= 0)
		{
			angle += Math.PI * 2;
		}
		
		return angle;
		
	}

	public double angleTo(double x2, double y2)
	{
		double angle = StrictMath.atan((y2-myPosition.getY())/(x2-myPosition.getX()));
		
		if((x2-myPosition.getX()) <= 0)
		{
			angle += Math.PI;
		}
		
		else if((y2-myPosition.getY()) <= 0)
		{
			angle += Math.PI * 2;
		}
		
		return angle;
	}
	
	public boolean isSelected()
	{
		return isSelected;
	}
	
	public void select()
	{
		isSelected = true;
	}
	
	public void unselect()
	{
		isSelected = false;
	}
	
	public Color getSelectedColor()
	{
		return selectedColor;
	}

}
