
/**
 * @author Ben Coleman
 *
 * This is a panel that provides for world control over the simulation.
 * Button functions:
 * run/pause: starts and stops the simulation
 * change number of objects: Changes the number of objects.  This also re-initializes all objects.
 * change temperature: Changes the global temperature of the simulation.  Also re-initializes all objects. Units: Kelvin
 * change container radius: Changes the radius of the circular container.  Also re-initializes all objects. Units: Angstoms
 * change seed: Changes the seed of the random number generator.  This only has an effect if you re-initialze the objects in
 * the container later.
 * speed: Adjusts the rate at which the simulation advances.  Each tick should roughly double the frame rate.
 * presets: Examples that have already been coded into the simulation
 * brownian motion: Launches another frame that lets the user select from 5 different examples that demonstrate brownian motion
 * one dimensional collisions: Launches another frame that lets the user select from 11 different one dimensional collisions.
 */
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.GridLayout;
import java.awt.BorderLayout;
import java.awt.Component;
import java.util.Hashtable;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JOptionPane;
import javax.swing.JSlider;
import javax.swing.JLabel;
import javax.swing.JFrame;
import javax.swing.Box;
import javax.swing.BoxLayout;

@SuppressWarnings("serial")
public class ControlPanel extends JPanel implements ActionListener, ChangeListener 
{
	private final int ABOUT_SIZE_X = 300;
	private final int ABOUT_SIZE_Y = 220;
	private final int ONE_SECOND = 1000;
	private final int MIN_FRAMES_PER_SECOND = 0;
	private final int MAX_FRAMES_PER_SECOND = 10;
	private final int INITIAL_SPEED = ONE_SECOND / (int)Math.pow(2, MAX_FRAMES_PER_SECOND);
	private final int HORIZONTAL_BORDER_PIXELS = 8;
	private final int VERTICAL_BORDER_PIXELS = 34;
	private final int BROWNIAN_SIZE_X = 254;
	private final int BROWNIAN_SIZE_Y = 165;
	private final int ONE_DIMENSION_SIZE_X = 380;
	private final int ONE_DIMENSION_SIZE_Y = 165;
	private final int PRESET_FRAME_X = 10;
	private final int PRESET_FRAME_Y = 25;
	private final int MAX_CONTAINER_RADIUS = 915;
	
	private boolean currentRunState;
	private boolean repaint;
	private boolean reset;
	private boolean runStateBeforeInput;
	private boolean sliderChangedBack;
	private int speed;
	private Simulation control;
	private JButton run;
	private JSlider framesPerSecondCoarse;
	private JFrame brownian;
	private JFrame oneDimensional;
	private JFrame aboutFrame;
	
	public ControlPanel(Simulation control)
	{
		this.control = control;
		currentRunState = false;
		runStateBeforeInput = false;
		repaint = false;
		sliderChangedBack = false;
		speed = INITIAL_SPEED;
		
		brownian = new JFrame("Brownian Motion");
		brownian.setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
		brownian.setSize(BROWNIAN_SIZE_X + HORIZONTAL_BORDER_PIXELS, BROWNIAN_SIZE_Y + VERTICAL_BORDER_PIXELS);
		brownian.setLocation(PRESET_FRAME_X, PRESET_FRAME_Y);
		
		oneDimensional = new JFrame("One-Dimensional Collisions");
		oneDimensional.setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
		oneDimensional.setSize(ONE_DIMENSION_SIZE_X + HORIZONTAL_BORDER_PIXELS, ONE_DIMENSION_SIZE_Y + VERTICAL_BORDER_PIXELS);
		oneDimensional.setLocation(PRESET_FRAME_X, PRESET_FRAME_Y);
		
		aboutFrame = new JFrame("About");
		aboutFrame.setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
		aboutFrame.setSize(ABOUT_SIZE_X + HORIZONTAL_BORDER_PIXELS, ABOUT_SIZE_Y + VERTICAL_BORDER_PIXELS);
		aboutFrame.setLocation(PRESET_FRAME_X, PRESET_FRAME_Y);
		
		BrownianPresets bPresets = new BrownianPresets(control, this);
		OneDimensionalPresets  oPresets = new OneDimensionalPresets(control, this);
		AboutPanel aPanel = new AboutPanel(this);
		
		brownian.setContentPane(bPresets);
		oneDimensional.setContentPane(oPresets);
		aboutFrame.setContentPane(aPanel);
		
		run = new JButton("run");
		run.setActionCommand("run");
		run.addActionListener(this);
		
		JButton objects = new JButton ("change number of objects");
		objects.setActionCommand("objects");
		objects.addActionListener(this);
		
		JButton temperature = new JButton ("change temperature");
		temperature.setActionCommand("temperature");
		temperature.addActionListener(this);
		
		JButton radius = new JButton ("change container radius");
		radius.setActionCommand("radius");
		radius.addActionListener(this);
		
		JButton seed = new JButton ("change seed");
		seed.setActionCommand("seed");
		seed.addActionListener(this);
		
		JButton reset = new JButton ("reset");
		reset.setActionCommand("reset");
		reset.addActionListener(this);
		
		JButton select = new JButton ("highlight an atom");
		select.setActionCommand("select");
		select.addActionListener(this);
		
		JButton about = new JButton ("About");
		about.setActionCommand("about");
		about.addActionListener(this);
		
		JLabel presetLabel = new JLabel("Presets", JLabel.CENTER);
		presetLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        
		JButton brownianPresets = new JButton ("Brownian Motion");
		brownianPresets.setActionCommand("brownian");
		brownianPresets.addActionListener(this);
		
		JButton oneDimensionalPresets = new JButton ("One-Dimensional Collisions");
		oneDimensionalPresets.setActionCommand("collisions");
		oneDimensionalPresets.addActionListener(this);
		
		
		JLabel sliderLabel = new JLabel("Speed", JLabel.CENTER);
        sliderLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        
        framesPerSecondCoarse = new JSlider(JSlider.HORIZONTAL, MIN_FRAMES_PER_SECOND, MAX_FRAMES_PER_SECOND, MAX_FRAMES_PER_SECOND);
        framesPerSecondCoarse.setMajorTickSpacing(1);
        framesPerSecondCoarse.addChangeListener(this);
        
        Hashtable<Integer, JLabel> labelTable1 = new Hashtable<Integer, JLabel>();
        labelTable1.put(new Integer(MIN_FRAMES_PER_SECOND), new JLabel("Slow"));
        labelTable1.put(new Integer(MAX_FRAMES_PER_SECOND), new JLabel("Fast"));
        framesPerSecondCoarse.setLabelTable(labelTable1);
        framesPerSecondCoarse.setPaintLabels(true);
        framesPerSecondCoarse.setPaintTicks(true);
        framesPerSecondCoarse.setSnapToTicks(true);
        
        
        JPanel subPanel1 = new JPanel(new GridLayout(0, 1));
		subPanel1.add(run);
		subPanel1.add(objects);
		subPanel1.add(temperature);
		subPanel1.add(radius);
		subPanel1.add(seed);
		subPanel1.add(reset);
		subPanel1.add(select);
		subPanel1.add(new JPanel());
		
		JPanel subPanel2 = new JPanel(new GridLayout(0, 1));
		subPanel2.add(sliderLabel);
		subPanel2.add(framesPerSecondCoarse);
		
		JPanel subPanel3 = new JPanel(new GridLayout(0, 1));
		subPanel3.add(new JPanel());
		subPanel3.add(presetLabel);
		subPanel3.add(brownianPresets);
		subPanel3.add(oneDimensionalPresets);
		subPanel3.add(new JPanel());
		subPanel3.add(about);
		
		Box panel = new Box(BoxLayout.Y_AXIS);
		panel.add(subPanel1);
		panel.add(subPanel2);
		panel.add(subPanel3);
		
		add(panel, BorderLayout.CENTER);
	}
	
	public boolean isRunning()
	{
		return currentRunState;
	}
	
	public void stopRunning()	//used by methods and classes that need to stop the simulation
	{
		currentRunState = false;
		run.setText("run");
	}
	
	public void startRunning()	//used by methods and classes that need to start the simulation
	{
		currentRunState = true;
		run.setText("pause");
	}
	
	public int getSpeed()
	{
		return speed;
	}
	
	@Override
	public void actionPerformed(ActionEvent arg0) 	//used by the buttons to perform the actions
	{
		SimulationGraphicsPanel.hideMenus();
		
		if(subFrameVisible())
		{
			JOptionPane.showMessageDialog(null, "Please select an option");
			return;
		}
		
		if(arg0.getActionCommand().equals("run"))
		{
			if(currentRunState)
			{
				run.setText("run");
			}
			
			else
			{
				run.setText("pause");
			}
			
			currentRunState = !currentRunState;
		}
		
		else if(arg0.getActionCommand().equals("objects"))
		{
			boolean temp = currentRunState;
			
			if(temp)
			{
				stopRunning();
			}
			
			SimulationGraphicsPanel.displayLoading();
			toggleRepaintRequest();
			int num = control.getNumAtoms();
			String str =  Main.guardedInput("Number of Atoms (> 0)" + "\n" +
											"Note: Running the simulation with more than 5000 atoms can cause severe performance problems",
											"" + num, false);
			
			if(str == null || str.equals(""))
			{
				SimulationGraphicsPanel.enableContainerRepaint();
				toggleRepaintRequest();
				
				if(temp)
				{
					startRunning();
				}
				
				return;
			}
			num = (int)Double.parseDouble(str);
			
			while(num <= 0 || num > Main.calculateNumCells(control.getContainerRadius()))
			{
				if(num <= 0)
				{
					JOptionPane.showMessageDialog(this, "Error: Please enter a value greater than 0", null, JOptionPane.ERROR_MESSAGE);
				}
				
				else
				{
					JOptionPane.showMessageDialog(this, "Error: Please enlarge your container to hold that many atoms", null, JOptionPane.ERROR_MESSAGE);
				}
				
				num = control.getNumAtoms();
				str =  Main.guardedInput("Number of Atoms (> 0)" +
										 "Note: Running the simulation with more than 5000 atoms can cause severe performance problems",
										 "" + num, false);
				
				if(str == null || str.equals(""))
				{
					SimulationGraphicsPanel.enableContainerRepaint();
					toggleRepaintRequest();
					
					if(temp)
					{
						startRunning();
					}
					
					return;
				}
				
				num = (int)Double.parseDouble(str);
				
			}

			Main.totalTime = 0;
			control.changeNumObjects(num);
			SimulationGraphicsPanel.enableContainerRepaint();
			toggleRepaintRequest();
		}
		
		else if(arg0.getActionCommand().equals("temperature"))
		{
			boolean temp = currentRunState;
			
			if(temp)
			{
				stopRunning();
			}
			
			SimulationGraphicsPanel.displayLoading();
			toggleRepaintRequest();
			double t = control.getTemperature();
			String str =  Main.guardedInput("New Temperature (Kelvins > 0)" + "\n" +
											"Note: Extremely high temperatures lead to choppy motion",
											"" + t, false);
			
			if(str == null || str.equals(""))
			{
				SimulationGraphicsPanel.enableContainerRepaint();
				toggleRepaintRequest();
				
				if(temp)
				{
					startRunning();
				}
				
				return;
			}
			
			t = Double.parseDouble(str);
			
			while(t <= 0)
			{
				JOptionPane.showMessageDialog(this, "Error: Please enter a value greater than 0", null, JOptionPane.ERROR_MESSAGE);
			 	t = control.getTemperature();
				str =  Main.guardedInput("New Temperature (Kelvins > 0)" +
										 "Note: Extremely high temperatures lead to choppy motion",
										 "" + t, false);
				
				if(str == null || str.equals(""))
				{
					SimulationGraphicsPanel.enableContainerRepaint();
					toggleRepaintRequest();
					
					if(temp)
					{
						startRunning();
					}
					
					return;
				}
				
				t = Double.parseDouble(str);
				
			}

			Main.totalTime = 0;
			control.changeTemperature(t);
			SimulationGraphicsPanel.enableContainerRepaint();
			toggleRepaintRequest();
		}
		
		else if(arg0.getActionCommand().equals("seed"))
		{
			long t = control.getSeed();
			String str =  Main.guardedInput("New Seed", "" + t, false);
			
			if(str == null || str.equals(""))
			{
				return;
			}
			
			t = (long)Double.parseDouble(str);
			
			control.setSeed(t);
		}
		
		else if(arg0.getActionCommand().equals("radius"))
		{
			boolean temp = currentRunState;
			
			if(temp)
			{
				stopRunning();
			}
			
			SimulationGraphicsPanel.displayLoading();
			toggleRepaintRequest();
			double t = control.getContainerRadius();
			String str =  Main.guardedInput("New Container Radius (Angstroms > 0, <= 915)", "" + (t * Math.pow(10, 10)), false);
			
			if(str == null || str.equals(""))
			{
				SimulationGraphicsPanel.enableContainerRepaint();
				toggleRepaintRequest();
				
				if(temp)
				{
					startRunning();
				}
				
				return;
			}
			
			t = Double.parseDouble(str) / Math.pow(10, 10);
			while(t <= 0 || t > MAX_CONTAINER_RADIUS || control.getNumAtoms() > Main.calculateNumCells(t))
			{
				if(t <= 0)
				{
					JOptionPane.showMessageDialog(this, "Error: Please enter a value greater than 0", null, JOptionPane.ERROR_MESSAGE);
				}
				
				else if(t > MAX_CONTAINER_RADIUS)
				{
					JOptionPane.showMessageDialog(this, "Error: Please enter a value less than or equal to 915", null, JOptionPane.ERROR_MESSAGE);
				}
				
				else
				{
					JOptionPane.showMessageDialog(this, "Error: Please enter a value that is large enought accomodate all of the atoms", null, JOptionPane.ERROR_MESSAGE);
				}
				
			 	t = control.getContainerRadius();
				str =  Main.guardedInput("New Container Radius (Angstroms > 0, <= 915)", "" + (t * Math.pow(10, 10)), false);
				
				if(str == null || str.equals(""))
				{
					SimulationGraphicsPanel.enableContainerRepaint();
					toggleRepaintRequest();
					
					if(temp)
					{
						startRunning();
					}
					
					return;
				}
				
				t = Double.parseDouble(str) / Math.pow(10, 10);
				
			}

			Main.totalTime = 0;
			control.setContainerRadius(t);
			SimulationGraphicsPanel.enableContainerRepaint();
			toggleRepaintRequest();
		}
		
		else if(arg0.getActionCommand().equals("reset"))
		{	
			stopRunning();
			SimulationGraphicsPanel.displayLoading();
			toggleRepaintRequest();
			
			try 
			{
				Thread.sleep(500);
			} 
			
			catch (InterruptedException e) 
			{
				e.printStackTrace();
			}
			
			reset = true;
		}
		
		else if(arg0.getActionCommand().equals("select"))
		{
			String str = Main.guardedInput("which atom?", "", false);
			
			if(str == null || str.equals(""))
			{
				return;
			}
			
			int num = (int)Double.parseDouble(str);
			
			while(num < 0 || num >= control.getObjects().size())
			{
				JOptionPane.showMessageDialog(this, "Error: please enter a value between 0 and " + (control.getObjects().size() -1) + ", inclusive", null, JOptionPane.ERROR_MESSAGE);
				str = Main.guardedInput("which atom?", "", false);
				
				if(str == null || str.equals(""))
				{
					return;
				}
				
				num = (int)Double.parseDouble(str);
			}
			
			control.getObjects().get(num).select();
			toggleRepaintRequest();
		}
		
		else if(arg0.getActionCommand().equals("brownian"))
		{
			runStateBeforeInput = currentRunState;
			
			if(runStateBeforeInput)
			{
				stopRunning();
			}
			
			brownian.setVisible(true);
		}
		
		else if(arg0.getActionCommand().equals("collisions"))
		{
			runStateBeforeInput = currentRunState;
			
			if(runStateBeforeInput)
			{
				stopRunning();
			}
			
			oneDimensional.setVisible(true);
		}
		
		else if(arg0.getActionCommand().equals("about"))
		{
			runStateBeforeInput = currentRunState;
			
			if(runStateBeforeInput)
			{
				stopRunning();
			}
			
			aboutFrame.setVisible(true);
		}
		
	}

	@Override
	public void stateChanged(ChangeEvent arg0)	//used by the slider to modify the speed
	{
		JSlider source = (JSlider)arg0.getSource();
		SimulationGraphicsPanel.hideMenus();
		
		if(!source.getValueIsAdjusting())
		{
			if(subFrameVisible() && !sliderChangedBack)
			{
				sliderChangedBack = true;
				
				if(speed == 0)
				{
					source.setValue(MAX_FRAMES_PER_SECOND);
				}
				
				else
				{
					source.setValue((int)(StrictMath.log(ONE_SECOND / speed) / StrictMath.log(2)));
				}
				
				JOptionPane.showMessageDialog(null, "Please select an option");
				return;
			}
			
			sliderChangedBack = false;
			speed = ONE_SECOND / (int)Math.pow(2, source.getValue());
		} 
		
	}
	
	public boolean needsRepaint()	//tells the main method when it needs the simulation display to be repainted
	{								//if the simulation is stopped
		return repaint;
	}
	
	public void toggleRepaintRequest()	//used by this class and other classes to signal that a repaint is needed
	{
		repaint = !repaint;
	}
	
	public boolean reset()	//tells the main method that the simulation needs a reset
	{
		return reset;
	}
	
	public void setReset(boolean b)
	{
		reset = b;
	}

	public void hasSelected(boolean cancel) 				//used to hide the sub frames for the examples
	{
		brownian.setVisible(false);
		oneDimensional.setVisible(false);
		aboutFrame.setVisible(false);
		
		if(runStateBeforeInput && cancel)
		{
			startRunning();
		}
		
		toggleRepaintRequest();
	}
	
	public boolean subFrameVisible()
	{
		return brownian.isVisible() || oneDimensional.isVisible() || aboutFrame.isVisible();
	}
	
}
