/**
 * @author original c++ code: Ng Weng Leong, Java port + improved event handling: Ben Coleman
 * 
 * This is the control for the simulation.  It provides event handling as well and stores the global variables
 * It stores the upcoming collisions in a PriorityQueue, then pulls them out when they come to the head of the queue
 *
 */
import java.util.ArrayList;
import java.util.Random;
import java.util.PriorityQueue;

public class Simulation 
{
	
	private final double BOLTZMANN_CONSTANT = 13806.50524 / Math.pow(10, 27);
	private final double MAX_GAP_TIME = 1e-23;
	private final double ATOM_START_GAP = 4e-12;
	private final double MAX_MOVE_TIME = 1;
	
	private boolean identicalMass;
	
	private double radius;
	private double mass;
	private double containerRadius;
	private double temperature;
	private double tot_mmtm_chg;
	
	private int n;
	
	private long seed;
	
	private Random randomGenerator;
	
	private ArrayList<Atom> atoms;
	
	private PriorityQueue<CollisionEvent> collisions;
	
	private CollisionEvent prev;
	
	private ControlPanel control;
	
	public Simulation(int n, double radius, double mass, double containerRadius, double temperature, long seed, ControlPanel panel)
	{		
		this.seed = seed;
		this.radius = radius;
		this.mass = mass;
		this.containerRadius = containerRadius;
		this.temperature = temperature;
		this.n = n;
		randomGenerator = new Random(seed);
		atoms = new ArrayList<Atom>();
		collisions = new PriorityQueue<CollisionEvent>();
		
		changeNumObjects(n);
		tot_mmtm_chg = 0; //this needs a better name
		prev = null;
		identicalMass = true;
		control = panel;
	}
	
	private void shuffleArrayList(ArrayList<Vec> arr)
	{
		for(int i = 0; i < arr.size() * 4; i ++)
		{
			int index1 = (int)(randomGenerator.nextDouble() * arr.size());
			int index2 = (int)(randomGenerator.nextDouble() * arr.size());
			Vec v = arr.get(index1);
			arr.set(index1, arr.get(index2));
			arr.set(index2, v);
		}
		
	}
	
	//finds the next collision then calculates it if it will happen within maxAllowedTime
	//if it does, then it checks the collision for validity
	//a collision is invalid if the numHits of at least one of the colliding atoms has changed
	//or the time to the next collision between the two atoms is greater than 1e-23
	//if the collision is invalid, it does one of two things depending on the state of the first atom
	//if the first Atom's numHits has changed, then the collision is tossed and nothing further is done
	//as the first atom will have calculated another collision for itself
	//if the numHits of the first Atom has not changed, then the other atom that the first was going to collide with
	//has been hit out of the way.  This also means that the first Atom has no known "good" events, so a new event for the 
	//first is calculated.
	//If the collision is valid, then new velocities are calculated and new collisions are generated for each Atom involved 
	//in the collision.
	//if the collision is invalid, the method advances the simulation forward by maxAllowed Time
	//it adds the time that it just advanced the simulation by then returns it
	public double update(double totalTime, double maxAllowedTime)
	{
		CollisionEvent event = collisions.peek();
		double timeToHit = event.getTime() - totalTime;
		
		if(timeToHit < maxAllowedTime)
		{
			//update all atom positions to the point of collision
			for(int i = 0; i < atoms.size(); i ++)
			{
				Vec position = atoms.get(i).getPosition();
				Vec velocity = atoms.get(i).getVelocity();
				Vec newPosition = position.add(velocity.scalarMultiplication(timeToHit));
				atoms.get(i).setPosition(newPosition);
			}
			
			totalTime += timeToHit;
			
			Atom one = event.getAtomOne();
			Atom two = event.getAtomTwo();

			//checks to see if the first Atom has hit anything since this collision was calculated
			if(one.getNumHits() == event.getHits())
			{
				//checks to see if the other member of the collision exists or has been hit out of the way
				//in addition, checks to see if the collision is identical to the previous one
				//(this is impossible in a collision between two Atoms, but this still occasionally happens)
				if(two != null && one.timeToHitAtom(two) < MAX_GAP_TIME && !event.equals(prev)) //atom-atom collision
				{
					Vec positionI = one.getPosition();
					Vec positionJ = two.getPosition();
					Vec unitNormal = positionJ.subtract(positionI);
					unitNormal.scalarDivisionEquals(StrictMath.sqrt(unitNormal.dotProduct(unitNormal)));
					Vec unitTangent = unitNormal.getOrthogonal();
					Vec velocityI = one.getVelocity();
					Vec velocityJ = two.getVelocity();
					double velocityINormal = unitNormal.dotProduct(velocityI);
					double velocityJNormal = unitNormal.dotProduct(velocityJ);
					double velocityITangent = unitTangent.dotProduct(velocityI);
					double velocityJTangent = unitTangent.dotProduct(velocityJ);
					double temp = velocityINormal;
					velocityINormal = (velocityINormal * (one.getMass() - two.getMass()) + 2.0 * two.getMass() * velocityJNormal) /
										(one.getMass() + two.getMass());
					velocityJNormal = (velocityJNormal * (two.getMass() - one.getMass()) + 2.0 * one.getMass() * temp) / (two.getMass() + one.getMass());
					velocityI = unitNormal.scalarMultiplication(velocityINormal).add(unitTangent.scalarMultiplication(velocityITangent));
					velocityJ = unitNormal.scalarMultiplication(velocityJNormal).add(unitTangent.scalarMultiplication(velocityJTangent));
					one.setVelocity(velocityI);
					two.setVelocity(velocityJ);
					one.iterateNumHits();
					two.iterateNumHits();
					updateCollisions(two, totalTime);
					LogUtility.printToLogFile("Time " + totalTime + ": Atoms " + one.getId() + ", " + two.getId() + " collided");
					prev = event;
				}
			
				else if(one.timeToHitContainer(containerRadius) < MAX_GAP_TIME)//atom-container collision
				{
					Vec pos = one.getPosition();
					Vec unitNormal = pos.scalarDivision(StrictMath.sqrt(pos.dotProduct(pos)));
					Vec unitTangent = unitNormal.getOrthogonal();
					Vec v = one.getVelocity();
					double normalVelocity = v.dotProduct(unitNormal);
					one.setVelocity(unitTangent.scalarMultiplication(v.dotProduct(unitTangent)).subtract(unitNormal.scalarMultiplication(normalVelocity)));
					tot_mmtm_chg += normalVelocity;
					one.iterateNumHits();
					LogUtility.printToLogFile("Time " + totalTime + ": Atom " + one.getId() + " collided with container");
					prev = event;
				}
				
				updateCollisions(one, totalTime);
			}
			
			collisions.remove();
		}
		
		//the next collision was going to occur too far in the future
		//positions are updated so that the simulation is advanced as far as the max time allows it
		else
		{
			for(int i = 0; i < atoms.size(); i ++)
			{
				Vec position = atoms.get(i).getPosition();
				Vec velocity = atoms.get(i).getVelocity();
				Vec newPosition = position.add(velocity.scalarMultiplication(maxAllowedTime));
				atoms.get(i).setPosition(newPosition);
			}
			
			totalTime += maxAllowedTime;
		}
		
		return totalTime;
	}
	
	//prints the statistics for the simulation:
	//pressure, total time, mean free path, mean time between collisions, and the Boltzmann constant calculated using data
	//from this simulation
	public void printStats(double totalTime)
	{
		double meanFreePath = 0;
		double meanFreeTime = 0;
		
		for(int i = 0;  i < atoms.size(); i ++)
		{
			int numHits = atoms.get(i).getNumHits();
			double pathLength = atoms.get(i).getPathLength();
			meanFreePath += pathLength / numHits;
			meanFreeTime += totalTime / numHits;
		}
		
		meanFreePath /= atoms.size();
		meanFreeTime /= atoms.size();
		double pressure = (tot_mmtm_chg * mass) / (totalTime * StrictMath.PI * containerRadius);
		double volume = StrictMath.PI * (StrictMath.pow(containerRadius, 2) - atoms.size() * StrictMath.pow(radius, 2));
		double experimentalBoltzmannConstant = pressure * volume / (atoms.size() * temperature);
		
		LogUtility.printToLogFile("Statistics");
		LogUtility.printToLogFile("----------");
		LogUtility.printToLogFile("Total time elapsed: " + totalTime + " s");
		LogUtility.printToLogFile("Mean Free Path = " + meanFreePath + " m");
		LogUtility.printToLogFile("Mean time between collisions = " + meanFreeTime + " s");
		LogUtility.printToLogFile("Pressure on container (P) = " + pressure + " kg/(s^2)");
		LogUtility.printToLogFile("Simulated Boltzmann constant = " + experimentalBoltzmannConstant
								  + " kg(m^2)/(K(s^2))");
	}
	
	public ArrayList<Atom> getObjects()
	{
		return atoms;
	}
	
	//used to ensure that the graphics displays the simulation advancing in a uniform fashion
	//advances the simulation by 1e-14 seconds
	//returns the new time for the simulation
	public double advance(double totalTime)
	{
		double stepTime = 1e-14;
		double finishTime = stepTime + totalTime;
		
		
		while(stepTime > 0 && control.isRunning())
		{
			totalTime = update(totalTime, stepTime);
			stepTime = finishTime - totalTime;
		}
		
		return totalTime;
	}
	
	//resets the simulation to the settings last entered
	public void reset()
	{
		setSeed(seed);
		changeNumObjects(atoms.size());
		tot_mmtm_chg = 0;
	}
	
	//re-initializes the simulation with newNum objects
	public void changeNumObjects(int newNum)
	{
		n = newNum;
		//partition container into one atom cells
		int ncc = (int)((containerRadius - radius)/( 2 * radius + ATOM_START_GAP)) + 1;
		double inc = (containerRadius - radius) / (ncc - 1);
		int numCells = 0;
		
		ArrayList<Vec> positions = new ArrayList<Vec>();
		
		
		for(int i = 0; i < ncc; i ++)
		{
			double ccr = i * inc;
			int nCells_cc = (int)(StrictMath.PI / StrictMath.atan(radius / ccr)) - 1;
			double theta = 2 * StrictMath.PI / nCells_cc;
			
			for(int j = 0; j < nCells_cc; j ++)
			{
				double dir = theta * j;
				Vec p = new Vec(ccr * StrictMath.sin(dir), ccr * StrictMath.cos(dir));
				positions.add(p);
			}
			
			numCells += nCells_cc;
		}
		
		shuffleArrayList(positions);
		atoms.clear();
		collisions.clear();
		
		//eqipartition of KE
		double speed = StrictMath.sqrt(2 * temperature * BOLTZMANN_CONSTANT / mass);

		for(int i = 0; i < n; i ++)
		{
			double theta = randomGenerator.nextDouble() * 2 * StrictMath.PI - StrictMath.PI;
			Vec v = new Vec(speed * StrictMath.sin(theta), speed * StrictMath.cos(theta));
			Atom a = new Atom(v, positions.get(i), mass, radius, i);
			atoms.add(a);
			updateCollisions(a, Main.totalTime);
		}
		
		identicalMass = true;
		
	}
	
	//changes the global temperature and resets the simulation so that the atom's velocity reflects the new temperature
	//this is used by ControlPanel to change the temperature
	public void changeTemperature(double newTemp)
	{
		temperature = newTemp;
		changeNumObjects(atoms.size());
	}

	public double getContainerRadius()
	{
		return containerRadius;
	}
	
	//changes the temperature but does not reset the atoms
	//used when the properties of an individual atom are changed
	public void setTemperature(double newTemp)
	{
		temperature = newTemp;
	}
	
	public double getTemperature()
	{
		return temperature;
	}
	
	public int getNumAtoms()
	{
		return atoms.size();
	}
	
	public double getAtomicRadius()
	{
		return radius;
	}
	
	public void setSeed(long s)
	{
		seed = s;
		randomGenerator.setSeed(s);
	}
	
	public long getSeed()
	{
		return seed;
	}
	
	//sets the container radius to a new value
	//re-initializes the atoms so no Atom is outside of the container.
	public void setContainerRadius(Double r)
	{
		containerRadius = r;
		changeNumObjects(atoms.size());
	}
	
	public void clearSim()
	{
		atoms.clear();
		clearCollisions();
	}
	
	//Adds the next collision for atm to the PriorityQueue.  currentTime is used to determine the time according to the global
	//clock when the collision will occur
	public void updateCollisions(Atom atm, double currentTime)
	{
		CollisionEvent event = new CollisionEvent(atm, null, atm.timeToHitContainer(containerRadius) + currentTime, atm.getNumHits());
		for(Atom atom: atoms)
		{
			double time = atm.timeToHitAtom(atom);
			
			if(atm != atom && time > 0 && time < MAX_MOVE_TIME)
			{
				CollisionEvent e = new CollisionEvent(atm, atom, time + currentTime, atm.getNumHits());
				
				if(e.compareTo(event) < 0)
				{
					event = e;
				}
				
			}
			
		}
		
		collisions.add(event);
		System.out.println(collisions.size());
	}
	
	public void clearCollisions()
	{
		collisions.clear();
	}
	
	public double meanFreePath()
	{
		double meanFreePath = 0;
		
		for(int i = 0;  i < atoms.size(); i ++)
		{
			int numHits = atoms.get(i).getNumHits();
			double pathLength = atoms.get(i).getPathLength();
			meanFreePath += pathLength / numHits;
		}
		
		meanFreePath /= atoms.size();
		return meanFreePath;
	}
	
	public double meanFreeTime(double totalTime)
	{
		double meanFreeTime = 0;
		
		for(int i = 0;  i < atoms.size(); i ++)
		{
			int numHits = atoms.get(i).getNumHits();
			meanFreeTime += totalTime / numHits;
		}
		
		meanFreeTime /= atoms.size();
		return meanFreeTime;
	}

	public double getBoltzmann(double totalTime) 
	{
		double volume = StrictMath.PI * (StrictMath.pow(containerRadius, 2) - atoms.size() * StrictMath.pow(radius, 2));
		double experimentalBoltzmannConstant = getPressure(totalTime) * volume / (atoms.size() * temperature);
		return experimentalBoltzmannConstant;
	}

	public double getPressure(double totalTime) 
	{
		if(identicalMass)
		{
			return (tot_mmtm_chg * mass) / (totalTime * StrictMath.PI * containerRadius);
		}
		
		else
		{
			return Double.NaN;
		}
		
	}
	
	public void notIdenticalMass()
	{
		double mass = atoms.get(0).getMass();
		
		for(Atom atm: atoms)
		{
			if(mass != atm.getMass())
			{
				identicalMass = false;
				return;
			}
			
		}
		
	}

	public void updateControlPanel(ControlPanel panel) {
		// TODO Auto-generated method stub
		control = panel;
	}
	
	//plot not included in this port

}
