
/**
 * @author Ben Coleman
 * This is the graphical representation of the simulation.  It also provides popup menus to allow the user
 * to modify the properties of an Atom and add a new Atom.  It assumes circular objects and a circular container.
 *
 */
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.Graphics;
import java.awt.Color;

import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;

@SuppressWarnings("serial")
public class SimulationGraphicsPanel extends JPanel implements MouseListener, ActionListener, KeyListener
{

	private final double BOLTZMANN_CONSTANT = 13806.50524 / Math.pow(10, 27);
	private final double HYDROGEN_MASS = 1.6736 / Math.pow(10, 27);	
	private final double HYDROGEN_RADIUS = 5.29e-11;
	private final double CALIBRATION_CYLINDER_RADIUS = 75;
	private final double CALIBRATION_ATOM_RADIUS = 2e-11;
	public static final int TEXT_AREA_LINES = 3;
	public static final int TEXT_AREA_LINE_HEIGHT = 18;
	public static final int TEXT_AREA_BOTTOM_BORDER = 7;
	private final int TEXT_AREA_PIXELS_FROM_SIDE = 5;
	private final int MIN_DISPLAY_RADIUS = 250;
	private final double MIN_SPEED = 1e-100;
	private final double DISTANCE_SCALE = Math.pow(10, 10);
	private final int FRAMES_BETWEEN_BACKGROUND_REPAINT = 25;
	
	private Simulation control;
	
	private static JPopupMenu menu;
	private static JPopupMenu menu2;
	
	private JMenuItem item1;
	private JMenuItem item2;
	private JMenuItem item3;
	private JMenuItem item4;
	private JMenuItem item5;
	private JMenuItem item6;
	private JMenuItem item7;
	private JMenuItem close1;
	private JMenuItem close2;
	
	private int selectedMolecule;
	private int displayRadius;
	private int repaintsUntilContainer;
	private int lastClickX;
	private int lastClickY;
	
	private double radius;
	private double scale;
	private double minRadius;
	
	private boolean mouseHasBeenPressed;
	private static boolean isStarting = true;
	private static boolean isLoading;
	
	private ControlPanel panel;
	
	public SimulationGraphicsPanel(Simulation control, ControlPanel panel)
	{
		isLoading = false;
		radius = control.getContainerRadius();
		minRadius = CALIBRATION_ATOM_RADIUS / (CALIBRATION_CYLINDER_RADIUS * HYDROGEN_RADIUS) * radius;
		mouseHasBeenPressed = false;
		repaintsUntilContainer = 0;
		selectedMolecule = 0;
		this.control = control;
		this.panel = panel;
		
		addKeyListener(this);
		addMouseListener(this);
		

		displayRadius = (getSize().height - TEXT_AREA_LINES * TEXT_AREA_LINE_HEIGHT) / 2;
		scale = displayRadius / radius;
		
		menu = new JPopupMenu();
		menu2 = new JPopupMenu();
		
		item1 = new JMenuItem("modify direction");
		item1.setActionCommand("dir");
		item1.setMnemonic(KeyEvent.VK_D);
		item1.addActionListener(this);
		item1.addKeyListener(this);
		menu.add(item1);
		
		item2 = new JMenuItem("modify velocity");
		item2.setActionCommand("velocity");
		item2.setMnemonic(KeyEvent.VK_V);
		item2.addActionListener(this);
		item2.addKeyListener(this);
		menu.add(item2);
		
		item3 = new JMenuItem("modify mass");
		item3.setActionCommand("mass");
		item3.setMnemonic(KeyEvent.VK_M);
		item3.addActionListener(this);
		item3.addKeyListener(this);
		menu.add(item3);
		
		item5 = new JMenuItem("modify radius");
		item5.setActionCommand("radius");
		item5.setMnemonic(KeyEvent.VK_R);
		item5.addActionListener(this);
		item5.addKeyListener(this);
		menu.add(item5);
		
		item6 = new JMenuItem("select / unselect");
		item6.setActionCommand("select");
		item6.setMnemonic(KeyEvent.VK_S);
		item6.addActionListener(this);
		item6.addKeyListener(this);
		menu.add(item6);
		
		item4 = new JMenuItem("get info");
		item4.setActionCommand("id");
		item4.setMnemonic(KeyEvent.VK_I);
		item4.addActionListener(this);
		item4.addKeyListener(this);
		menu.add(item4);
		
		item7 = new JMenuItem("place atom here");
		item7.setActionCommand("atom");
		item7.setMnemonic(KeyEvent.VK_A);
		item7.addActionListener(this);
		item7.addKeyListener(this);
		menu2.add(item7);
		
		close1 = new JMenuItem("close");
		close1.setActionCommand("close");
		close1.setMnemonic(KeyEvent.VK_C);
		close1.addActionListener(this);
		close1.addKeyListener(this);
		menu.add(close1);
		
		close2 = new JMenuItem("close");
		close2.setActionCommand("close");
		close2.setMnemonic(KeyEvent.VK_C);
		close2.addActionListener(this);
		close2.addKeyListener(this);
		menu2.add(close2);
		
		menu.pack();
		menu.addKeyListener(this);
	}
	
	//used for launching the menus
	//launches them with either the right mouse button or middle mouse button
	@Override
	public void mouseClicked(MouseEvent arg0)
	{
		if(panel.subFrameVisible())
		{
			JOptionPane.showMessageDialog(null, "Please select an option");
			return;
		}
		
		menu.setVisible(false);
		menu2.setVisible(false);
		
		if(!panel.isRunning() && (arg0.getButton() == MouseEvent.BUTTON2 || arg0.getButton() == MouseEvent.BUTTON3))
		{
			selectedMolecule = 0;
			
			while(selectedMolecule < control.getObjects().size() &&
				 (control.getObjects().get(selectedMolecule).distanceTo(convertToSimCoordinates(arg0.getX(), true),
																		convertToSimCoordinates(arg0.getY(), false))
				  > minRadius && 
				  control.getObjects().get(selectedMolecule).distanceTo(convertToSimCoordinates(arg0.getX(), true),
																		convertToSimCoordinates(arg0.getY(), false))
				  > control.getObjects().get(selectedMolecule).getRadius()))
			{
				selectedMolecule ++;
			}
			
			if(selectedMolecule < control.getObjects().size())
			{
				menu.setLocation(arg0.getPoint());
				menu.setLabel("Atom " + control.getObjects().get(selectedMolecule).getId()); //does nothing at the moment
				menu.setVisible(true);
			}
			
			else
			{
				lastClickX = arg0.getX();
				lastClickY = arg0.getY();
				menu2.setLocation(arg0.getPoint());
				menu2.setVisible(true);
			}
			
		}
		
	}

	@Override
	public void mouseEntered(MouseEvent arg0) 
	{
		// not used

	}

	@Override
	public void mouseExited(MouseEvent arg0) 
	{
		// not used
	}

	//used for moving Atoms
	//activates on the left mouse button
	//stops the simulation if it is running (any button)
	//restarts it with a left click not on an Atom
	@Override
	public void mousePressed(MouseEvent arg0) 
	{
		if(panel.subFrameVisible())
		{
			JOptionPane.showMessageDialog(null, "Please select an option");
			return;
		}
		
		if(!panel.isRunning() && arg0.getButton() == MouseEvent.BUTTON1)
		{
			selectedMolecule = 0;
			
			while(selectedMolecule < control.getObjects().size() && 
				 (control.getObjects().get(selectedMolecule).distanceTo(convertToSimCoordinates(arg0.getX(), true),
						  												convertToSimCoordinates(arg0.getY(), false))
				  > minRadius && 
				  control.getObjects().get(selectedMolecule).distanceTo(convertToSimCoordinates(arg0.getX(), true),
																		convertToSimCoordinates(arg0.getY(), false))
				  > control.getObjects().get(selectedMolecule).getRadius()))
			{
				selectedMolecule ++;
			}
			
			if(selectedMolecule < control.getObjects().size())
			{
				mouseHasBeenPressed = true;
				control.getObjects().get(selectedMolecule).select();
				requestRepaint();
			}
			
			else
			{
				panel.startRunning();
			}
			
		}
		
		else
		{
			panel.stopRunning();
		}
		
	}

	//if an Atom was selected by the MousePressed method, then this method will move the Atom
	//to the location the mouse was released at if the Atom fits
	@Override
	public void mouseReleased(MouseEvent e) 
	{
		if(!panel.isRunning() && mouseHasBeenPressed && selectedMolecule < control.getObjects().size())
		{
			Atom obj = control.getObjects().get(selectedMolecule);
			obj.unselect();
			
			if(isValidLocation(convertToSimCoordinates(e.getX(), true), convertToSimCoordinates(e.getY(), false)))
			{
				obj.setPosition(new Vec(convertToSimCoordinates(e.getY(), false),
						        		convertToSimCoordinates(e.getX(), true)));
			}

			requestRepaint();
			control.updateCollisions(obj, Main.totalTime);
			mouseHasBeenPressed = false;
		}
		
	}
	
	//draws the simulation
	//every 25 times that this is called, it will repaint the background grey
	//it draws all Atoms blue and the container black
	//if the simulation is redoing the world, then it will inform the user that this is happening
	//automatically syncs the painting area with the y coordinate of the Panel
	public void paintComponent(Graphics g)
	{
		displayRadius = (getSize().height - TEXT_AREA_LINES * TEXT_AREA_LINE_HEIGHT - TEXT_AREA_BOTTOM_BORDER) / 2;
		
		if(displayRadius < MIN_DISPLAY_RADIUS)
		{
			displayRadius = MIN_DISPLAY_RADIUS;
		}
		
		updateScale();
		g.setColor(Color.black);
		menu.setVisible(false);
		menu2.setVisible(false);
		
		if(isStarting)
		{			
			g.setColor(Color.gray);
			g.fillRect(0, 0, displayRadius * 2, displayRadius * 2);
			
			isStarting = false;
		}

		g.setColor(Color.black);
		g.fillOval(0, 0, displayRadius * 2, displayRadius * 2);
		
		g.setColor(Color.blue);
		
		for(int i = 0; i < control.getObjects().size(); i ++)
		{
			Atom obj = control.getObjects().get(i);
			
			if(obj.isSelected())
			{
				g.setColor(obj.getSelectedColor());
			}
			
			Vec v = obj.getPosition();
			
			int objXSize = (int)(obj.getRadius() * 2 * scale), 
				objYSize = (int)(obj.getRadius() * 2 * scale);
			
			if(objXSize < 1)
			{
				objXSize = 1;
			}
				
			if(objYSize < 1)
			{
				objYSize = 1;
			}
			
			g.fillOval(convertToGraphicsCoordinates(v.getX() - obj.getRadius(), true), 
					   convertToGraphicsCoordinates(v.getY() + obj.getRadius(), false),
					   objXSize, objYSize);
			g.setColor(Color.blue);
		}
		
		g.setColor(Color.white);
		g.fillRect(0, displayRadius * 2, displayRadius * 2, TEXT_AREA_LINES * TEXT_AREA_LINE_HEIGHT +TEXT_AREA_BOTTOM_BORDER);
		g.setColor(Color.black);
		double displayTime = Math.round(Main.totalTime * Math.pow(10, 13)) / 10.0;
		
		//g.setFont(new Font(Font.MONOSPACED, Font.PLAIN, 10));
		
		//line 1
		String str = "";
		
		if(isLoading)
		{
			str = "Please wait, generating world";
			isLoading = false;
		}
		
		else
		{
			str = "Temperature: " + (Math.round(control.getTemperature() * 100) / 100.0) + 
			 		" K, Number of Atoms: " + control.getNumAtoms() + 
			 		", Elapsed Time: " + displayTime + "E-12 s";
		}
			
		g.drawChars(str.toCharArray(), 0, str.length(),
				  	TEXT_AREA_PIXELS_FROM_SIDE, displayRadius * 2 + TEXT_AREA_LINE_HEIGHT);
		//line 2
		str = "";
		
		if(!isLoading)
		{
			str = "Mean Free Path: " + (Math.round(control.meanFreePath() / 1e-15) / 1000.0) + 
			 	  "E-12 m, Mean time between collisions: " + (Math.round(control.meanFreeTime(Main.totalTime) / 1e-18) / 1000.0) + "E-15 s";
		}
		
		g.drawChars(str.toCharArray(), 0, str.length(), 
					TEXT_AREA_PIXELS_FROM_SIDE, displayRadius * 2 + TEXT_AREA_LINE_HEIGHT * 2);
		//line 3
		str = "";
					
		if(!isLoading)
		{
			str = "Pressure: " + (Math.round(control.getPressure(Main.totalTime) * 1e8) / 100.0) + 
			 	  "E-6 kg/(s^2), Simulated Boltzmann Constant: " + (Math.round(control.getBoltzmann(Main.totalTime) * 1e27) / 10000.0) + 
			 	  "E-23 kg(m^2)/(K(s^2))";
		}
					
		g.drawChars(str.toCharArray(), 0, str.length(), 
					TEXT_AREA_PIXELS_FROM_SIDE, displayRadius * 2 + TEXT_AREA_LINE_HEIGHT * 3);
		
		repaintsUntilContainer ++;
		
		if(repaintsUntilContainer % FRAMES_BETWEEN_BACKGROUND_REPAINT == 0)
		{
			isStarting = true;
		}
		
	}

	//handles events from the menus
	@Override
	public void actionPerformed(ActionEvent e) 
	{
		if(panel.subFrameVisible())
		{
			JOptionPane.showMessageDialog(null, "Please select an option");
			return;
		}
		
		menu.setVisible(false);
		menu2.setVisible(false);
		Atom obj = null;
		
		if(selectedMolecule < control.getObjects().size())
		{
			obj = control.getObjects().get(selectedMolecule);
		}
		
		if(e.getActionCommand().equals("dir"))
		{
			String str = Main.guardedInput("New direction? (radians)", "" + obj.getDirection(), false);
			
			if(str == null || str.equals(""))
			{
				return;
			}
			
			double dir = Double.parseDouble(str);
			double velocity = StrictMath.sqrt(StrictMath.pow(obj.getVelocity().getX(), 2) + 
							  StrictMath.pow(obj.getVelocity().getY(), 2));
			Vec newVelocity = new Vec(velocity * StrictMath.sin(dir), velocity * StrictMath.cos(dir));
			obj.setVelocity(newVelocity);
			control.updateCollisions(obj, Main.totalTime);
		}
		
		else if(e.getActionCommand().equals("velocity"))
		{
			String str = Main.guardedInput("New speed? (m/s)", "" + obj.getLinearSpeed(), false);
			
			if(str == null || str.equals(""))
			{
				return;
			}
			
			Vec newVelocity = obj.getVelocity();
			newVelocity = newVelocity.unitVector();
			double speed = Double.parseDouble(str);
			
			//If two Atoms have zero velocity, NaN could be returned by the timeToHitAtom method
			//this causes problems where all atoms appear to fly off
			//this sets the the speed to a value that is low enough so that it is for all practical purposes, zero
			if(speed < MIN_SPEED && speed >= 0)
			{
				speed = MIN_SPEED;
			}
			
			else if(speed > -MIN_SPEED && speed <= 0)
			{
				speed = -MIN_SPEED;
			}
			
			newVelocity = newVelocity.scalarMultiplication(speed);
			obj.setVelocity(newVelocity);
			
			double kineticEnergy = 0;
			
			for(Atom obj2: control.getObjects())
			{
				kineticEnergy += StrictMath.pow(obj2.getLinearSpeed(), 2) * obj2.getMass() / 2;
			}
			
			kineticEnergy /= control.getObjects().size();
			control.setTemperature(kineticEnergy / BOLTZMANN_CONSTANT);
			Main.totalTime = 0;
			requestRepaint();
			
			for(Atom atm: control.getObjects())
			{
				control.updateCollisions(atm, Main.totalTime);
			}
			
		}
		
		else if(e.getActionCommand().equals("mass"))
		{
			double mass = obj.getMass();
			String str = Main.guardedInput("New mass? (Atomic Mass Units)", "" + mass / HYDROGEN_MASS, false);
			
			if(str == null || str.equals(""))
			{
				return;
			}
			
			mass = Double.parseDouble(str) * HYDROGEN_MASS;
			
			while(mass <= 0)
			{
				JOptionPane.showMessageDialog(this, "Error: please enter a value greater than 0", null, JOptionPane.ERROR_MESSAGE);
				mass = obj.getMass();
				str = Main.guardedInput("New mass? (Atomic Mass Units)", "" + mass / HYDROGEN_MASS, false);
				
				if(str == null || str.equals(""))
				{
					return;
				}
				
				mass = Double.parseDouble(str) * HYDROGEN_MASS;
				
			}
			
			obj.setMass(mass);
			double kineticEnergy = 0;
			
			for(Atom obj2: control.getObjects())
			{
				kineticEnergy += StrictMath.pow(obj2.getLinearSpeed(), 2) * obj2.getMass() / 2;
			}
			
			kineticEnergy /= control.getObjects().size();
			control.setTemperature(kineticEnergy / BOLTZMANN_CONSTANT);
			control.notIdenticalMass();
			Main.totalTime = 0;
			
			for(Atom atm: control.getObjects())
			{
				control.updateCollisions(atm, Main.totalTime);
			}
			
			requestRepaint();
		}
		
		else if(e.getActionCommand().equals("radius"))
		{
			double radius = obj.getRadius();
			String str = Main.guardedInput("new radius? (Angstroms)", "" + radius * DISTANCE_SCALE, false);

			if(str == null || str.equals(""))
			{
				return;
			}
			
			radius = Double.parseDouble(str) / DISTANCE_SCALE;
			
			while(radius <= 0 || 
				  StrictMath.sqrt(StrictMath.pow(obj.getPosition().getX(), 2) + StrictMath.pow(obj.getPosition().getY(), 2))
						  > control.getContainerRadius() - radius)
			{ 
				if(radius <= 0)
				{
					JOptionPane.showMessageDialog(this, "Error: please enter a value greater than 0", null, JOptionPane.ERROR_MESSAGE);
				}
				
				else
				{
					JOptionPane.showMessageDialog(this, "Error: the atom 's boundaries exceed the container wall", 
												  null, JOptionPane.ERROR_MESSAGE);
				}
				radius = obj.getRadius();
				str = Main.guardedInput("new radius? (Angstroms)", "" + radius * DISTANCE_SCALE, false);

				if(str == null || str.equals(""))
				{
					return;
				}
				
				radius = Double.parseDouble(str) / DISTANCE_SCALE;
			}
			
			obj.setRadius(radius);
			requestRepaint();
			Main.totalTime = 0;
			
			for(Atom atm: control.getObjects())
			{
				control.updateCollisions(atm, Main.totalTime);
			}
			
		}
		
		else if(e.getActionCommand().equals("select"))
		{
			if(obj.isSelected())
			{
				obj.unselect();
			}
			
			else
			{
				obj.select();
			}
			
			requestRepaint();
		}
		
		if(e.getActionCommand().equals("id"))
		{
			JOptionPane.showMessageDialog(this, obj.toString(), "Object Info", JOptionPane.PLAIN_MESSAGE);
		}
		
		if(e.getActionCommand().equals("atom"))
		{
			double radius = HYDROGEN_RADIUS;
			String str = Main.guardedInput("radius? (Angstroms)", "" + radius * DISTANCE_SCALE, false);

			if(str == null || str.equals(""))
			{
				return;
			}
			
			radius = Double.parseDouble(str) / DISTANCE_SCALE;
			
			while(radius <= 0)
			{
				JOptionPane.showMessageDialog(this, "Error: please enter a value greater than 0", null, JOptionPane.ERROR_MESSAGE);
				radius = HYDROGEN_RADIUS;
				str = Main.guardedInput("new radius? (Angstroms)", "" + radius * DISTANCE_SCALE, false);

				if(str == null || str.equals(""))
				{
					return;
				}
				
				radius = Double.parseDouble(str) / DISTANCE_SCALE;
			}
			
			double x = convertToSimCoordinates(lastClickX, true),
				   y = convertToSimCoordinates(lastClickY, false);
			
			if(isValidLocation(x, y, radius))
			{
				double mass = HYDROGEN_MASS;
				str = Main.guardedInput("Mass? (Atomic Mass Units)", "" + mass / HYDROGEN_MASS, false);
				
				if(str == null || str.equals(""))
				{
					return;
				}
				
				mass = Double.parseDouble(str) * HYDROGEN_MASS;
				
				while(mass <= 0)
				{
					JOptionPane.showMessageDialog(this, "Error: please enter a value greater than 0", null, JOptionPane.ERROR_MESSAGE);
					mass = HYDROGEN_MASS;
					str = Main.guardedInput("Mass? (Atomic Mass Units)", "" + mass / HYDROGEN_MASS, false);
					
					if(str == null || str.equals(""))
					{
						return;
					}
					
					mass = Double.parseDouble(str) * HYDROGEN_MASS;
					
				}

				double defaultVelocity = StrictMath.sqrt(2 * control.getTemperature() * BOLTZMANN_CONSTANT / mass);
				
				str = Main.guardedInput("New speed? (m/s)", "" + defaultVelocity, false);
				
				double speed = defaultVelocity;
				
				if(str == null || str.equals(""))
				{
					return;
				}
				
				speed = Double.parseDouble(str);
				
				if(speed < MIN_SPEED && speed >= 0)
				{
					speed = MIN_SPEED;
				}
				
				else if(speed > -MIN_SPEED && speed <= 0)
				{
					speed = -MIN_SPEED;
				}
				
				str = Main.guardedInput("New direction? (radians)", "" + 0, false);

				double dir = 0;
				
				if(str == null || str.equals(""))
				{
					return;
				}
				
				dir = Double.parseDouble(str);
				
				Atom a = new Atom(new Vec(StrictMath.sin(dir) * speed, StrictMath.cos(dir) * speed), 
								  new Vec(y, x), mass, radius, control.getObjects().size());
				control.getObjects().add(a);
				
				double kineticEnergy = 0;
				
				for(Atom obj2: control.getObjects())
				{
					kineticEnergy += StrictMath.pow(obj2.getLinearSpeed(), 2) * obj2.getMass() / 2;
				}
				
				kineticEnergy /= control.getObjects().size();
				control.setTemperature(kineticEnergy / BOLTZMANN_CONSTANT);
				Main.totalTime = 0;
				
				for(Atom atm: control.getObjects())
				{
					control.updateCollisions(atm, Main.totalTime);
				}
				
				control.notIdenticalMass();
			}
			
			else
			{
				JOptionPane.showMessageDialog(null, "Error: radius was too large", null, JOptionPane.ERROR_MESSAGE);
			}
			
			requestRepaint();
		}
		
		else if(e.getActionCommand().equals("close"))
		{
			menu.setVisible(false);
			menu2.setVisible(false);
		}
		
	}
	
	public void requestRepaint()
	{
		panel.toggleRepaintRequest();
	}
	
	//checks if the selected atom can fit in the new location
	private boolean isValidLocation(double x, double y)
	{
		if(StrictMath.sqrt(StrictMath.pow(x, 2)
			 + StrictMath.pow(y, 2))
		   > control.getContainerRadius() - control.getObjects().get(selectedMolecule).getRadius())
		{
			return false;
		}
		
		for(int i = 0; i < control.getObjects().size(); i ++)
		{
			Atom obj = control.getObjects().get(i);
			
			if(obj.getId() != control.getObjects().get(selectedMolecule).getId()
			   && StrictMath.sqrt(StrictMath.pow((obj.getPosition().getX() - x), 2) + 
					              StrictMath.pow((obj.getPosition().getY() - y), 2))
			 	  < control.getObjects().get(selectedMolecule).getRadius() + obj.getRadius())
			{
				return false;
			}
			
		}
		
		return true;
	}
	
	//checks a given location to see if an object with the given radius can fit there without overlapping anything
	public boolean isValidLocation(double x, double y, double radius)
	{
		if(StrictMath.sqrt(StrictMath.pow(x, 2) + StrictMath.pow(y, 2))
			   > control.getContainerRadius() - radius)
			{
				return false;
			}
			
			for(int i = 0; i < control.getObjects().size(); i ++)
			{
				Atom obj = control.getObjects().get(i);
				
				if(StrictMath.sqrt(StrictMath.pow((obj.getPosition().getX() - x), 2) + 
						           StrictMath.pow((obj.getPosition().getY() - y), 2))
				 	  < radius + obj.getRadius())
				{
					return false;
				}
				
			}
			
			return true;
	}
	
	public int convertToGraphicsCoordinates(double coordinate, boolean isX)
	{
		double newCoordinate = coordinate;
		
		if(!isX)
		{
			newCoordinate = - newCoordinate;
		}
		
		newCoordinate = newCoordinate * scale + displayRadius;
		return (int)Math.round(newCoordinate);
	}
	
	public double convertToSimCoordinates(int coordinate, boolean isX)
	{
		double newCoordinate = coordinate;
		newCoordinate = (newCoordinate - displayRadius) / scale;
		
		if(!isX)
		{
			newCoordinate = - newCoordinate;
		}
		
		return newCoordinate;
	}

	//not used
	@Override
	public void keyPressed(KeyEvent arg0) {
		
	}

	//not used
	@Override
	public void keyReleased(KeyEvent e) {
		
	}

	//This does not work.  If it was functional, it would allow the user to use the keyboard to control the popup menus.
	@Override
	public void keyTyped(KeyEvent e) 
	{
		menu.setVisible(false);
		menu2.setVisible(false);
		
		Atom obj = control.getObjects().get(selectedMolecule);
		int ch = e.getKeyCode();
		
		if(ch == KeyEvent.VK_D)
		{
			double dir = Double.parseDouble(JOptionPane.showInputDialog("new direction? (radians)"));
			double velocity = StrictMath.sqrt(StrictMath.pow(obj.getVelocity().getX(), 2) + 
							    			  StrictMath.pow(obj.getVelocity().getY(), 2));
			Vec newVelocity = new Vec(velocity * StrictMath.sin(dir), velocity * StrictMath.cos(dir));
			obj.setVelocity(newVelocity);
		}
		
		if(ch == KeyEvent.VK_V)
		{
			Vec newVelocity = obj.getVelocity();
			newVelocity = newVelocity.unitVector();
			newVelocity = newVelocity.scalarMultiplication(Double.parseDouble(JOptionPane.showInputDialog("new speed (m/s)")));
			obj.setVelocity(newVelocity);
			
			double kineticEnergy = 0;
			
			for(Atom obj2: control.getObjects())
			{
				kineticEnergy += StrictMath.pow(obj2.getLinearSpeed(), 2) * obj2.getMass() / 2;
			}
			
			kineticEnergy /= control.getObjects().size();
			control.setTemperature(kineticEnergy / BOLTZMANN_CONSTANT);
		}
		
		if(ch == KeyEvent.VK_M)
		{
			obj.setMass(Double.parseDouble(JOptionPane.showInputDialog(this, "new mass(kg)")));
			
			double kineticEnergy = 0;
			
			for(Atom obj2: control.getObjects())
			{
				kineticEnergy += StrictMath.pow(obj2.getLinearSpeed(), 2) * obj2.getMass() / 2;
			}
			
			kineticEnergy /= control.getObjects().size();
			control.setTemperature(kineticEnergy / BOLTZMANN_CONSTANT);
		}
		
		if(ch == KeyEvent.VK_R)
		{
			obj.setRadius(Double.parseDouble(JOptionPane.showInputDialog(this, "new radius(m)")));
			requestRepaint();
		}
		
		if(ch == KeyEvent.VK_S)
		{
			if(obj.isSelected())
			{
				obj.unselect();
			}
			
			else
			{
				obj.select();
			}
			
			requestRepaint();
		}
		
		if(ch == KeyEvent.VK_I)
		{
			JOptionPane.showMessageDialog(this, obj.toString(), "Object Info", JOptionPane.PLAIN_MESSAGE);
		}
		
	}
	
	public void updateScale()
	{
		radius = control.getContainerRadius();
		scale = displayRadius / radius;
	}
	
	//enables painting of the grey background
	public static void enableContainerRepaint()
	{
		isStarting = true;
	}
	
	//enables the display of the loading method at the bottom right
	public static void displayLoading()
	{
		isLoading = true;
	}

	public static void disableLoading() 
	{
		isLoading = false;
	}
	
	public static void hideMenus()
	{
		menu.setVisible(false);
		menu2.setVisible(false);
	}
	
}
